library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity man_clk is
    Port ( rst_in : in STD_LOGIC;
           clk_in : in STD_LOGIC;
           man_in : in STD_LOGIC;
           man_out : out STD_LOGIC);
end man_clk;

architecture Behavioral of man_clk is

    signal count_next, count_r : integer range 0 to 2000000;  -- range to count for debouncing
    signal stable_signal : STD_LOGIC;
    signal edge_r, edge_next : STD_LOGIC;
    
begin

register_update: process (clk_in, rst_in) is
begin
    if (rst_in = '1') then
        count_r <= 0;
        edge_r <= '0';
    elsif (rising_edge(clk_in)) then
        count_r <= count_next;
        edge_r <= edge_next;
    end if;
end process register_update;

debouncer_logic: process (man_in, count_r) is
begin
    -- Default values
    count_next <= count_r;
    stable_signal <= '0';

    if man_in = '1' then
        if count_r < 2000000 then
            count_next <= count_r + 1;
        else
            count_next <= 2000000;
        end if;
    else
      count_next <= 0;
    end if;

    if count_r = 2000000 then
        stable_signal <= '1';
    end if;
    
end process debouncer_logic;

edge_detector_logic:process(edge_r, stable_signal)
begin
    edge_next <= edge_r;
        
    case edge_r is
    when '0' =>
        if(stable_signal = '1') then
            edge_next <= '1';
        end if;
    when others =>
        if(stable_signal = '0') then
            edge_next <= '0';
        end if;
    end case;    
end process edge_detector_logic;

man_out <= (not edge_r) and stable_signal;

end Behavioral;
